@@ -1,6 +1,6 @@
 // $Header$
 //
-// Copyright (C) 2000 - 2004, by
+// Copyright (C) 2000 - 2007, by
 // 
 // Carlo Wood, Run on IRC <carlo@alinoe.com>
 // RSA-1024 0x624ACAD5 1997-01-26                    Sign & Encrypt
@@ -55,14 +55,11 @@ struct rtld_global {
 #ifdef LIBCWD_DEBUGBFD
 #include <iomanip>
 #endif
-#if CWDEBUG_ALLOC
-#include <elf.h>		// Elf32_Off
-#endif
 #include "cwd_debug.h"
 #include "ios_base_Init.h"
 #include "exec_prog.h"
 #include "match.h"
-#include "elf32.h"
+#include "elfxx.h"
 #include "cwd_bfd.h"
 #include <libcwd/class_object_file.h>
 #include <libcwd/private_string.h>
@@ -76,7 +73,9 @@ namespace libcwd {
 #if LIBCWD_THREAD_SAFE && CWDEBUG_ALLOC && __GNUC__ == 3 && __GNUC_MINOR__ == 4
     extern char* pool_allocator_lock_symbol_ptr;
 #endif
+#if CWDEBUG_ALLOC
     extern void remove_type_info_references(object_file_ct const* object_file_ptr LIBCWD_COMMA_TSD_PARAM);
+#endif
   } // namespace _private_
 
     // New debug channel
@@ -163,7 +162,7 @@ static bool const statically_linked = true;
       }
 
       // cwbfd::
-      bfile_ct* NEEDS_READ_LOCK_find_object_file(elf32::bfd_st const* abfd)
+      bfile_ct* NEEDS_READ_LOCK_find_object_file(elfxx::bfd_st const* abfd)
       {
 	object_files_ct::const_iterator i(NEEDS_READ_LOCK_object_files().begin());
 	for(; i != NEEDS_READ_LOCK_object_files().end(); ++i)
@@ -174,10 +173,10 @@ static bool const statically_linked = true;
 
       // cwbfd::
       struct symbol_less {
-	bool operator()(elf32::asymbol_st const* a, elf32::asymbol_st const* b) const;
+	bool operator()(elfxx::asymbol_st const* a, elfxx::asymbol_st const* b) const;
       };
 
-      bool symbol_less::operator()(elf32::asymbol_st const* a, elf32::asymbol_st const* b) const
+      bool symbol_less::operator()(elfxx::asymbol_st const* a, elfxx::asymbol_st const* b) const
       {
 	if (a == b)
 	  return false;
@@ -255,12 +254,13 @@ static bool const statically_linked = true;
         _private_::internal_string realpath_str;
 	char const* dbgfilename;
 
-	for (int dbg = 0; dbg < 3; ++dbg)
+	for (int dbg = 0; dbg < 4; ++dbg)
 	{
 	  // This loop has dbgfilename_str run over the the values:
 	  // 0) /path/libfoo.so.3
 	  // 1) /path/.debug/libfoo.so.3
 	  // 2) /usr/lib/debug/path/libfoo.so.3
+	  // 3) /usr/lib/debug/libfoo.so.3
 	  // where /path/libfoo.so.3 is the file that filename
 	  // points to (if it's a symlink).
 
@@ -327,7 +327,7 @@ static bool const statically_linked = true;
 		delete [] linkname;
 	      }
 	    }		// Next symbolic link.
-	    // Store the resolved real path, we need it for dbg == 1 and dbg == 2.
+	    // Store the resolved real path, we need it for dbg == 1, dbg == 2 and dbg == 3.
 	    realpath_str = dbgfilename_str;
 	  }
 	  else if (dbg == 1)
@@ -341,6 +341,8 @@ static bool const statically_linked = true;
 	      continue;
 	    dbgfilename_str = "/usr/lib/debug" + realpath_str;
 	  }
+	  else if (dbg == 3)
+	    dbgfilename_str = "/usr/lib/debug/" + realpath_str.substr(realpath_str.find_last_of('/') + 1);
 
 	  dbgfilename = dbgfilename_str.c_str();
 
@@ -359,10 +361,10 @@ static bool const statically_linked = true;
 	  }
 
 	  // Store the previous values - if this try doesn't work out, we want to use the old values.
-	  elf32::bfd_st* old_M_abfd = M_abfd;
+	  elfxx::bfd_st* old_M_abfd = M_abfd;
 	  long old_storage_needed = storage_needed;
 
-	  M_abfd = elf32::bfd_st::openr(dbgfilename);
+	  M_abfd = elfxx::bfd_st::openr(dbgfilename);
 
 	  if (M_abfd->check_format())
 	  {
@@ -407,16 +409,16 @@ static bool const statically_linked = true;
 	M_abfd->M_s_end_offset = 0;
 	M_abfd->object_file = this;
 
-	M_symbol_table = (elf32::asymbol_st**) malloc(storage_needed);	// LEAK12
+	M_symbol_table = (elfxx::asymbol_st**) malloc(storage_needed);	// LEAK12
 	M_number_of_symbols = M_abfd->canonicalize_symtab(M_symbol_table);
 
 	if (M_number_of_symbols > 0)
 	{
-#if LIBCWD_THREAD_SAFE && CWDEBUG_ALLOC && __GNUC__ == 3 && __GNUC_MINOR__ == 4
-	  Elf32_Off S_lock_value = 0;
-#endif
 #if CWDEBUG_ALLOC
-	  Elf32_Off exit_funcs = 0;
+#if LIBCWD_THREAD_SAFE && __GNUC__ == 3 && __GNUC_MINOR__ == 4
+	  elfxx::Elfxx_Off S_lock_value = 0;
+#endif
+	  elfxx::Elfxx_Off exit_funcs = 0;
 #endif
 	  size_t s_end_offset = M_abfd->M_s_end_offset;
 	  if (!s_end_offset && M_number_of_symbols > 0)
@@ -476,11 +478,11 @@ static bool const statically_linked = true;
 	      start_values_map_ct start_values;
 	      unsigned int best_count = 0;
 	      void* best_start = 0;
-	      for (elf32::asymbol_st** s = M_symbol_table; s <= &M_symbol_table[M_number_of_symbols - 1]; ++s)
+	      for (elfxx::asymbol_st** s = M_symbol_table; s <= &M_symbol_table[M_number_of_symbols - 1]; ++s)
 	      {
 		if ((*s)->name == 0 || ((*s)->flags & BSF_FUNCTION) == 0 || ((*s)->flags & (BSF_GLOBAL|BSF_WEAK)) == 0)
 		  continue;
-		elf32::asection_st const* sect = (*s)->section;
+		elfxx::asection_st const* sect = (*s)->section;
 		if (sect->name[1] == 't' && !strcmp(sect->name, ".text"))
 		{
 #if CWDEBUG_ALLOC
@@ -582,8 +584,8 @@ static bool const statically_linked = true;
           BFD_ACQUIRE_WRITE_LOCK;	// Needed for M_function_symbols.
 
 	  // Throw away all symbols that are not a global variable or function, store the rest in a vector.
-	  elf32::asymbol_st** se2 = &M_symbol_table[M_number_of_symbols - 1];
-	  for (elf32::asymbol_st** s = M_symbol_table; s <= se2;)
+	  elfxx::asymbol_st** se2 = &M_symbol_table[M_number_of_symbols - 1];
+	  for (elfxx::asymbol_st** s = M_symbol_table; s <= se2;)
 	  {
 #if LIBCWD_THREAD_SAFE && CWDEBUG_ALLOC && __GNUC__ == 3 && __GNUC_MINOR__ == 4
 	    if (is_libstdcpp && strcmp((*s)->name,
@@ -620,7 +622,7 @@ static bool const statically_linked = true;
 	  {
 	    // M_function_symbols is a set<> that sorts in reverse order (using symbol_key_greater)!
 	    // So use begin() here in order to get the last symbol.
-	    elf32::asymbol_st const* last_symbol = (*M_function_symbols.begin()).get_symbol();
+	    elfxx::asymbol_st const* last_symbol = (*M_function_symbols.begin()).get_symbol();
 	    if (symbol_size(last_symbol) == 100001)
 	    {
 	      BFD_RELEASE_WRITE_LOCK;
@@ -633,7 +635,7 @@ static bool const statically_linked = true;
 #endif
 	      BFD_ACQUIRE_WRITE_LOCK;
 	    }
-	    elf32::asymbol_st const* first_symbol = (*M_function_symbols.rbegin()).get_symbol();
+	    elfxx::asymbol_st const* first_symbol = (*M_function_symbols.rbegin()).get_symbol();
 	    M_start = symbol_start_addr(first_symbol); // Use the lowest start address of all symbols.
 	    if (s_end_start_addr)
 	      M_size = (char*)s_end_start_addr - (char*)M_start;
@@ -643,7 +645,7 @@ static bool const statically_linked = true;
 	      {
 		M_start_last_symbol = symbol_start_addr(last_symbol);	// Initialize M_start_last_symbol.
 		M_size = (char*)last_symbol->section->vma + last_symbol->section->M_size - (char*)M_start;
-		symbol_size(const_cast<elf32::asymbol_st*>(last_symbol)) = M_size - ((char*)symbol_start_addr(last_symbol) - (char*)M_start);
+		symbol_size(const_cast<elfxx::asymbol_st*>(last_symbol)) = M_size - ((char*)symbol_start_addr(last_symbol) - (char*)M_start);
 	      }
 	      else
 		M_size = (char*)symbol_start_addr(last_symbol) + symbol_size(last_symbol) - (char*)M_start;
@@ -665,7 +667,7 @@ static bool const statically_linked = true;
 #endif
 	    for(function_symbols_ct::iterator i(M_function_symbols.begin()); i != M_function_symbols.end(); ++i)
 	    {
-	      elf32::asymbol_st const* s = i->get_symbol();
+	      elfxx::asymbol_st const* s = i->get_symbol();
 	      Dout(dc::always, s->name << " (" << s->section->name << ") = " << s->value);
 	    }
 #if CWDEBUG_ALLOC
@@ -771,7 +773,9 @@ static bool const statically_linked = true;
 	LIBCWD_ASSERT( __libcwd_tsd.rdlocked_by1[object_files_instance] != __libcwd_tsd.tid && __libcwd_tsd.rdlocked_by2[object_files_instance] != __libcwd_tsd.tid );
 #endif
 #endif
+#if CWDEBUG_ALLOC
 	_private_::remove_type_info_references(&M_object_file LIBCWD_COMMA_TSD);
+#endif
 	LIBCWD_DEFER_CANCEL;
 	BFD_ACQUIRE_WRITE_LOCK;
 	set_alloc_checking_off(LIBCWD_TSD);
@@ -887,7 +891,7 @@ static bool const statically_linked = true;
 	      }
 	      else if (pid_token == 0 && token == "PID")
 		pid_token = current_token;
-	      else if (command_token == 0 && (token == "COMMAND") || (token == "CMD"))
+	      else if ((command_token == 0 && (token == "COMMAND")) || (token == "CMD"))
 	      {
 		command_token = current_token;
 		command_column = current_column;
@@ -1047,7 +1051,7 @@ static bool const statically_linked = true;
       {
 	LIBCWD_TSD_DECLARATION;
 	for (char const* p = buf; p < &buf[len]; ++p)
-	  if (p[0] == '=' && p[1] == '>' && p[2] == ' ' || p[2] == '\t')
+	  if ((p[0] == '=' && p[1] == '>' && p[2] == ' ') || p[2] == '\t')
 	  {
 	    p += 2;
 	    while (*p == ' ' || *p == '\t')
@@ -1055,7 +1059,7 @@ static bool const statically_linked = true;
 	    if (*p != '/' && *p != '.')
 	      break;
 	    char const* q;
-	    for (q = p; q < &buf[len] && *q > ' '; ++q);
+	    for (q = p; q < &buf[len] && *q > ' '; ++q) ;
 	    if (*q == '\n')	// This ldd doesn't return an offset (ie, on solaris).
 	    {
 	      set_alloc_checking_off(LIBCWD_TSD);
@@ -1101,7 +1105,7 @@ static bool const statically_linked = true;
 		     std::setw(6) << "Size" << ' ' <<
 		     std::resetiosflags(std::ios::right);
 	std::cout << "Name value flags\n";
-	elf32::asymbol_st** symbol_table = object_file->get_symbol_table();
+	elfxx::asymbol_st** symbol_table = object_file->get_symbol_table();
 	for (long n = object_file->get_number_of_symbols() - 1; n >= 0; --n)
 	{
 	  std::cout << std::setiosflags(std::ios::left) <<
@@ -1339,8 +1343,8 @@ static bool const statically_linked = true;
       {
 	if (object_file)
 	{
-	  elf32::asymbol_st dummy_symbol;		// A dummy symbol with size 1 and start `addr',
-	  elf32::asection_st dummy_section;
+	  elfxx::asymbol_st dummy_symbol;		// A dummy symbol with size 1 and start `addr',
+	  elfxx::asection_st dummy_section;
 
 	  // Make symbol_start_addr(&dummy_symbol) and symbol_size(&dummy_symbol) return the correct value:
 	  dummy_symbol.bfd_ptr = object_file->get_bfd();
@@ -1351,7 +1355,7 @@ static bool const statically_linked = true;
 	  function_symbols_ct::iterator i(object_file->get_function_symbols().find(symbol_ct(&dummy_symbol)));
 	  if (i != object_file->get_function_symbols().end())
 	  {
-	    elf32::asymbol_st const* p = (*i).get_symbol();
+	    elfxx::asymbol_st const* p = (*i).get_symbol();
 	    if (addr < symbol_start_addr(p) + symbol_size(p))
 	      return &(*i);
 	  }
@@ -1521,6 +1525,8 @@ typedef location_ct bfd_location_ct;
       if (!object_file && !statically_linked)
       {
         // Try to load everything again... previous loaded libraries are skipped based on load address.
+	int possible_object_files = 0;
+	bfile_ct* possible_object_file;
 	for(link_map* l = *dl_loaded_ptr; l; l = l->l_next)
 	{
 	  bool already_loaded = false;
@@ -1545,6 +1551,11 @@ typedef location_ct bfd_location_ct;
 		    (void*)((char*)((*iter)->get_start()) + (*iter)->size()));
 #endif
 		already_loaded = true;
+		if ((*iter)->get_lbase() <= addr && (char*)(*iter)->get_start() + (*iter)->size() > addr)
+		{
+		  ++possible_object_files;
+		  possible_object_file = *iter;
+		}
 		break;
 	      }
 	    }
@@ -1560,6 +1571,14 @@ typedef location_ct bfd_location_ct;
 	BFD_ACQUIRE_WRITE2READ_LOCK;
 	object_file = NEEDS_READ_LOCK_find_object_file(addr);
         BFD_RELEASE_READ_LOCK;
+	// This can happen when address is in a function that is not visible
+	// as function from the ELF sections, and is outside the detected
+	// range determined by functions that are. It means that there
+	// are not debug symbols for this object file, otherwise it can't
+	// happen. Anyway, it's better to make this guess than to print
+	// <unknown object file>.
+	if (!object_file && possible_object_files == 1)
+	  object_file = possible_object_file;
       }
 #endif
       LIBCWD_RESTORE_CANCEL;
@@ -1578,9 +1597,9 @@ typedef location_ct bfd_location_ct;
       symbol_ct const* symbol = pc_symbol(addr, object_file);
       if (symbol)
       {
-	elf32::asymbol_st const* p = symbol->get_symbol();
-	elf32::bfd_st* abfd = p->bfd_ptr;
-	elf32::asection_st const* sect = p->section;
+	elfxx::asymbol_st const* p = symbol->get_symbol();
+	elfxx::bfd_st* abfd = p->bfd_ptr;
+	elfxx::asection_st const* sect = p->section;
 	char const* file;
 	LIBCWD_ASSERT( object_file->get_bfd() == abfd );
 	set_alloc_checking_off(LIBCWD_TSD);
@@ -1610,7 +1629,7 @@ typedef location_ct bfd_location_ct;
 	    static int const BSF_WARNING_PRINTED = 0x40000000;
 	    if (!M_object_file->has_no_debug_line_sections() && !(p->flags & BSF_WARNING_PRINTED))
 	    {
-	      const_cast<elf32::asymbol_st*>(p)->flags |= BSF_WARNING_PRINTED;
+	      const_cast<elfxx::asymbol_st*>(p)->flags |= BSF_WARNING_PRINTED;
 	      set_alloc_checking_off(LIBCWD_TSD);
 	      {
 		_private_::internal_string demangled_name;		// Alloc checking must be turned off already for this string.
