@@ -1,4 +1,4 @@
-// $Header: /cvsroot/libcwd/libcwd/function.cc,v 1.6 2004/08/19 17:28:32 libcw Exp $
+// $Header$
 //
 // Copyright (C) 2003 - 2004, by
 // 
@@ -124,8 +124,7 @@ void Function::M_init(char const* expr, unsigned int flags)
       {
 	static unsigned int const setflags = BSF_FUNCTION;
 	symbol_ct const& symbol(*i2);
-	if (symbol.is_defined() &&
-	    (symbol.get_symbol()->flags & setflags) == setflags)
+	if ((symbol.get_symbol()->flags & setflags) == setflags)
 	{
 	  bool matched = false;
 	  char const* name = symbol.get_symbol()->name;
